#ifndef __SconeDATA__
#define __SconeDATA__
/*****************************************************************************
 * Copyright (C) 2009-2020   this file is part of the NPTool Project       *
 *                                                                           *
 * For the licensing terms see $NPTOOL/Licence/NPTool_Licence                *
 * For the list of contributors see $NPTOOL/Licence/Contributors             *
 *****************************************************************************/

/*****************************************************************************
 * Original Author: Pierre Morfouace  contact address: pierre.morfouace2@cea.fr                        *
 *                                                                           *
 * Creation Date  : March 2020                                           *
 * Last update    :                                                          *
 *---------------------------------------------------------------------------*
 * Decription:                                                               *
 *  This class hold Scone Raw data                                    *
 *                                                                           *
 *---------------------------------------------------------------------------*
 * Comment:                                                                  *
 *                                                                           *   
 *                                                                           *
 *****************************************************************************/

// STL
#include <vector>
using namespace std;

// ROOT
#include "TObject.h"

class TSconeData : public TObject {
  //////////////////////////////////////////////////////////////
  // data members are hold into vectors in order 
  // to allow multiplicity treatment
  private: 
    // Energy
    vector<UShort_t>   fScone_E_DetectorNbr;
    vector<UShort_t>   fScone_E_PlasticNbr;
    vector<Double_t>   fScone_Energy;

    // Time
    vector<UShort_t>   fScone_T_DetectorNbr;
    vector<UShort_t>   fScone_T_PlasticNbr;
    vector<Double_t>   fScone_Time;


  //////////////////////////////////////////////////////////////
  // Constructor and destructor
  public: 
    TSconeData();
    ~TSconeData();
    

  //////////////////////////////////////////////////////////////
  // Inherited from TObject and overriden to avoid warnings
  public:
    void Clear();
    void Clear(const Option_t*) {};
    void Dump() const;


  //////////////////////////////////////////////////////////////
  // Getters and Setters
  // Prefer inline declaration to avoid unnecessary called of 
  // frequently used methods
  // add //! to avoid ROOT creating dictionnary for the methods
  public:
    //////////////////////    SETTERS    ////////////////////////
    // Energy
    inline void SetEnergy(const UShort_t& DetNbr,const UShort_t& PlasticNbr, const Double_t& Energy){
      fScone_E_DetectorNbr.push_back(DetNbr);
      fScone_E_PlasticNbr.push_back(PlasticNbr);
      fScone_Energy.push_back(Energy);
    };//!

    // Time
    inline void SetTime(const UShort_t& DetNbr,const UShort_t& PlasticNbr, const Double_t& Time)	{
      fScone_T_DetectorNbr.push_back(DetNbr);     
      fScone_T_PlasticNbr.push_back(PlasticNbr);     
      fScone_Time.push_back(Time);
    };//!


    //////////////////////    GETTERS    ////////////////////////
    // Energy
    inline UShort_t GetMultEnergy() const
      {return fScone_E_DetectorNbr.size();}
    inline UShort_t GetE_DetectorNbr(const unsigned int &i) const 
      {return fScone_E_DetectorNbr[i];}//!
    inline UShort_t GetE_PlasticNbr(const unsigned int &i) const 
      {return fScone_E_PlasticNbr[i];}//!
    inline Double_t Get_Energy(const unsigned int &i) const 
      {return fScone_Energy[i];}//!

    // Time
    inline UShort_t GetMultTime() const
      {return fScone_T_DetectorNbr.size();}
    inline UShort_t GetT_DetectorNbr(const unsigned int &i) const 
      {return fScone_T_DetectorNbr[i];}//!
    inline UShort_t GetT_PlasticNbr(const unsigned int &i) const 
      {return fScone_T_PlasticNbr[i];}//!
    inline Double_t Get_Time(const unsigned int &i) const 
      {return fScone_Time[i];}//!


  //////////////////////////////////////////////////////////////
  // Required for ROOT dictionnary
  ClassDef(TSconeData,1)  // SconeData structure
};

#endif
